@@ -5,6 +5,7 @@ import json
 import os
 
 import datasets
+from datasets.tasks import TextClassification
 
 
 _CITATION = """\
@@ -65,6 +66,7 @@ class AllocineDataset(datasets.GeneratorBasedBuilder):
             supervised_keys=None,
             homepage="https://github.com/TheophileBlard/french-sentiment-analysis-with-bert",
             citation=_CITATION,
+            task_templates=[TextClassification(text_column="review", label_column="label")],
         )
 
     def _split_generators(self, dl_manager):
