@@ -41,33 +41,14 @@ _URL = "https://u.cs.biu.ac.il/~koppel/BlogCorpus.htm"
 _DATA_URL = "http://www.cs.biu.ac.il/~koppel/blogs/blogs.zip"
 
 
-class BlogAuthorshipCorpusConfig(datasets.BuilderConfig):
-    """BuilderConfig for BlogAuthorship."""
-
-    def __init__(self, data_url, **kwargs):
-        """BuilderConfig for BlogAuthorship
-
-        Args:
-          data_url: `string`, url to the dataset (word or raw level)
-          **kwargs: keyword arguments forwarded to super.
-        """
-        super(BlogAuthorshipCorpusConfig, self).__init__(
-            version=datasets.Version(
-                "1.0.0",
-            ),
-            **kwargs,
-        )
-        self.data_url = data_url
-
-
 class BlogAuthorshipCorpus(datasets.GeneratorBasedBuilder):
     """TODO(BlogAuthorship): Short description of my dataset."""
 
-    VERSION = datasets.Version("0.1.0")
+    VERSION = datasets.Version("1.0.0")
     BUILDER_CONFIGS = [
-        BlogAuthorshipCorpusConfig(
-            name="blog-authorship-corpus",
-            data_url=_DATA_URL,
+        datasets.BuilderConfig(
+            name="blog_authorship_corpus",
+            version=VERSION,
             description="word level dataset. No processing is needed other than replacing newlines with <eos> tokens.",
         ),
     ]
@@ -98,71 +79,62 @@ class BlogAuthorshipCorpus(datasets.GeneratorBasedBuilder):
 
     def _split_generators(self, dl_manager):
         """Returns SplitGenerators."""
-        if self.config.name == "blog-authorship-corpus":
-            data = dl_manager.download_and_extract(self.config.data_url)
-            data_dir = os.path.join(data, "blogs")
-            files = sorted(glob.glob(os.path.join(data_dir, "*.xml")))
-            train_files = []
-            validation_files = []
+        data = dl_manager.download_and_extract(_DATA_URL)
+        data_dir = os.path.join(data, "blogs")
+        files = sorted(glob.glob(os.path.join(data_dir, "*.xml")))
+        train_files = []
+        validation_files = []
 
-            for i, file_path in enumerate(files):
-                # 95% / 5% (train / val) split
-                if i % 20 == 0:
-                    validation_files.append(file_path)
-                else:
-                    train_files.append(file_path)
+        for i, file_path in enumerate(files):
+            # 95% / 5% (train / val) split
+            if i % 20 == 0:
+                validation_files.append(file_path)
+            else:
+                train_files.append(file_path)
 
-            return [
-                datasets.SplitGenerator(
-                    name=datasets.Split.TRAIN,
-                    gen_kwargs={"files": train_files, "split": "train"},
-                ),
-                datasets.SplitGenerator(
-                    name=datasets.Split.VALIDATION,
-                    gen_kwargs={"files": validation_files, "split": "validation"},
-                ),
-            ]
-        else:
-            raise ValueError("{} does not exist".format(self.config.name))
+        return [
+            datasets.SplitGenerator(
+                name=datasets.Split.TRAIN,
+                gen_kwargs={"files": train_files, "split": "train"},
+            ),
+            datasets.SplitGenerator(
+                name=datasets.Split.VALIDATION,
+                gen_kwargs={"files": validation_files, "split": "validation"},
+            ),
+        ]
 
     def _generate_examples(self, files, split):
         def parse_date(line):
             # parse line to date
             return line.strip().split("<date>")[-1].split("</date>")[0]
 
+        key = 0
         for file_path in files:
-            counter = 0
             file_name = os.path.basename(file_path)
             logger.info("generating examples from = %s", file_path)
             file_id, gender, age, job, horoscope = tuple(file_name.split(".")[:-1])
+            # TODO: yield also file_id?
 
             # Note: import xml.etree.ElementTree as etree does not work. File cannot be parsed
             # use open instead
-            with open(file_path, encoding="utf-8") as f:
-                # some files are corrupted, so have to work with python`s try here
-                try:
-                    date = ""
-                    for line in f:
-                        line = line.strip()
-                        if "<date>" in line:
-                            date = parse_date(line)
-                        elif line != "" and not line.startswith("<"):
-                            # need sub_id to be certain that no tf_records is identical
-                            sub_id = counter
-                            counter += 1
-                            if date == "":
-                                logger.warning("Date missing for {} in {}".format(line, file_name))
-                            assert date is not None, "Date is missing before {}".format(line)
-                            blog = {
-                                "text": line,
-                                "date": date,
-                                "gender": gender,
-                                "age": int(age),
-                                "job": job,
-                                "horoscope": horoscope,
-                            }
-                            yield "{}_{}_{}".format(file_id, sub_id, date), blog
-                        else:
-                            continue
-                except UnicodeDecodeError as e:
-                    logger.warning("{} cannot be loaded. Error message: {}".format(file_path, e))
+            with open(file_path, encoding="latin_1") as f:
+                date = ""
+                for line in f:
+                    line = line.strip()
+                    if "<date>" in line:
+                        date = parse_date(line)
+                    elif line != "" and not line.startswith("<"):
+                        if date == "":
+                            logger.warning("Date missing for {} in {}".format(line, file_name))
+                        assert date is not None, "Date is missing before {}".format(line)
+                        yield key, {
+                            "text": line,
+                            "date": date,
+                            "gender": gender,
+                            "age": int(age),
+                            "job": job,
+                            "horoscope": horoscope,
+                        }
+                        key += 1
+                    else:
+                        continue
