@@ -14,14 +14,11 @@
 # limitations under the License.
 """The IndoNLU benchmark is a collection of resources for training, evaluating, and analyzing natural language understanding systems for Bahasa Indonesia"""
 
-from __future__ import absolute_import, division, print_function
 
 import ast
 import csv
 import textwrap
 
-import six
-
 import datasets
 
 
@@ -535,12 +532,10 @@ class Indonlu(datasets.GeneratorBasedBuilder):
         if self.config.name in (sentence_features + ner_ + pos_):
             features = {
                 text_feature: datasets.Sequence(datasets.Value("string"))
-                for text_feature in six.iterkeys(self.config.text_features)
+                for text_feature in self.config.text_features.keys()
             }
         else:
-            features = {
-                text_feature: datasets.Value("string") for text_feature in six.iterkeys(self.config.text_features)
-            }
+            features = {text_feature: datasets.Value("string") for text_feature in self.config.text_features}
 
         if self.config.label_classes:
             if self.config.name in sentence_features:
@@ -576,7 +571,7 @@ class Indonlu(datasets.GeneratorBasedBuilder):
         ]
 
     def _generate_examples(self, filepath):
-        """ Yields examples. """
+        """Yields examples."""
         csv_file = ["emot", "wrete", "facqa", "casa", "hoasa"]
         tsv_file = ["smsa"]
         txt_file = ["terma", "keps"]
