@@ -20,6 +20,7 @@ import csv
 import os
 
 import datasets
+from datasets.tasks import TextClassification
 
 
 _DOWNLOAD_URL = "https://raw.githubusercontent.com/timpal0l/swedish-sentiment/main/swedish_sentiment.zip"
@@ -50,6 +51,7 @@ class SwedishReviews(datasets.GeneratorBasedBuilder):
             supervised_keys=None,
             homepage="https://github.com/timpal0l/swedish-sentiment",
             citation=_CITATION,
+            task_templates=[TextClassification(text_column="text", label_column="label")],
         )
 
     def _split_generators(self, dl_manager):
