@@ -199,6 +199,7 @@ submissions and assigned them a CEFR level."""
         else:
             assert False
 
+        id_ = 0
         for level in levels:
             with open(filepath / f"{level}.{split}.json", "r", encoding="utf-8") as fp:
                 for line in fp:
@@ -218,4 +219,5 @@ submissions and assigned them a CEFR level."""
                     if self.config.name == "wi":
                         out["userid"] = o.get("userid", "")
 
-                    yield o["id"], out
+                    yield id_, out
+                    id_ += 1
