@@ -33,6 +33,7 @@
 import os
 
 import datasets
+from datasets.tasks import TextClassification
 
 
 _DESCRIPTION = """\
@@ -123,6 +124,7 @@ class YelpPolarity(datasets.GeneratorBasedBuilder):
             supervised_keys=None,
             homepage="https://course.fast.ai/datasets",
             citation=_CITATION,
+            task_templates=[TextClassification(text_column="text", label_column="label")],
         )
 
     def _vocab_text_gen(self, train_file):
