@@ -47,11 +47,15 @@ class GenerateMode(enum.Enum):
 
     The generations modes:
 
+    +------------------------------------+-----------+---------+
     |                                    | Downloads | Dataset |
-    | -----------------------------------|-----------|---------|
+    +====================================+===========+=========+
     | `REUSE_DATASET_IF_EXISTS` (default)| Reuse     | Reuse   |
+    +------------------------------------+-----------+---------+
     | `REUSE_CACHE_IF_EXISTS`            | Reuse     | Fresh   |
+    +------------------------------------+-----------+---------+
     | `FORCE_REDOWNLOAD`                 | Fresh     | Fresh   |
+    +------------------------------------+-----------+---------+
     """
 
     REUSE_DATASET_IF_EXISTS = "reuse_dataset_if_exists"
@@ -59,7 +63,7 @@ class GenerateMode(enum.Enum):
     FORCE_REDOWNLOAD = "force_redownload"
 
 
-class DownloadManager(object):
+class DownloadManager:
     def __init__(
         self,
         dataset_name: Optional[str] = None,
@@ -107,7 +111,9 @@ class DownloadManager(object):
             raise ValueError("You need to specify 'temp_location' in PipelineOptions to upload files")
 
         def upload(local_file_path):
-            remote_file_path = os.path.join(remote_dir, "downloads", os.path.basename(local_file_path))
+            remote_file_path = os.path.join(
+                remote_dir, config.DOWNLOADED_DATASETS_DIR, os.path.basename(local_file_path)
+            )
             logger.info(
                 "Uploading {} ({}) to {}.".format(
                     local_file_path, size_str(os.path.getsize(local_file_path)), remote_file_path
@@ -142,7 +148,7 @@ class DownloadManager(object):
             downloaded_path(s): `str`, The downloaded paths matching the given input
                 url_or_urls.
         """
-        cache_dir = self._download_config.cache_dir or os.path.join(config.HF_DATASETS_CACHE, "downloads")
+        cache_dir = self._download_config.cache_dir or config.DOWNLOADED_DATASETS_PATH
         max_retries = self._download_config.max_retries
 
         def url_to_downloaded_path(url):
@@ -284,3 +290,14 @@ class DownloadManager(object):
 
     def get_recorded_sizes_checksums(self):
         return self._recorded_sizes_checksums.copy()
+
+    def delete_extracted_files(self):
+        paths_to_delete = set(self.extracted_paths.values()) - set(self.downloaded_paths.values())
+        for key, path in list(self.extracted_paths.items()):
+            if path in paths_to_delete and os.path.isfile(path):
+                os.remove(path)
+                del self.extracted_paths[key]
+
+    def manage_extracted_files(self):
+        if self._download_config.delete_extracted:
+            self.delete_extracted_files()
