@@ -30,7 +30,7 @@ from .version import Version
 logger = get_logger(__name__)
 
 
-class MockDownloadManager(object):
+class MockDownloadManager:
     dummy_file_name = "dummy_data"
     datasets_scripts_dir = "datasets"
 
@@ -201,3 +201,9 @@ class MockDownloadManager(object):
             # while now we expected the dummy_data.zip file to be a directory containing
             # the downloaded file.
             return path_to_dummy_data
+
+    def delete_extracted_files(self):
+        pass
+
+    def manage_extracted_files(self):
+        pass
